@@ -101,12 +101,20 @@ class Subcommands(object):
                  {'flag': ["-s", "--sfn"],
                   'help': "tibanna step function name (e.g. 'tibanna_unicorn_monty'); " +
                           "your current default is %s)" % TIBANNA_DEFAULT_STEP_FUNCTION_NAME,
-                  'default': TIBANNA_DEFAULT_STEP_FUNCTION_NAME}],
+                  'default': TIBANNA_DEFAULT_STEP_FUNCTION_NAME},
+                 {'flag': ["-z", "--soft"],
+                  'help': "instead of directly killing the execution, " +
+                          "send abort signal to s3 so that step function can handle it",
+                  'action': "store_true"}],
             'kill_all':
                 [{'flag': ["-s", "--sfn"],
                   'help': "tibanna step function name (e.g. 'tibanna_unicorn_monty'); " +
                           "your current default is %s)" % TIBANNA_DEFAULT_STEP_FUNCTION_NAME,
-                  'default': TIBANNA_DEFAULT_STEP_FUNCTION_NAME}],
+                  'default': TIBANNA_DEFAULT_STEP_FUNCTION_NAME},
+                 {'flag': ["-z", "--soft"],
+                  'help': "instead of directly killing the execution, " +
+                          "send abort signal to s3 so that step function can handle it",
+                  'action': "store_true"}],
             'log':
                 [{'flag': ["-e", "--exec-arn"],
                   'help': "execution arn of the specific job to log"},
@@ -163,6 +171,8 @@ class Subcommands(object):
                   'help': "use a specified ebs type for the rerun (gp2 vs io1)"},
                  {'flag': ["-p", "--ebs-iops"],
                   'help': "use a specified ebs iops for the rerun"},
+                 {'flag': ["-w", "--ebs-throughput"],
+                  'help': "use a specified ebs throughput (for gp3 type EBS) for the rerun"},
                  {'flag': ["-k", "--key-name"],
                   'help': "use a specified key name for the rerun"},
                  {'flag': ["-n", "--name"],
@@ -214,6 +224,8 @@ class Subcommands(object):
                   'help': "use a specified ebs type for the rerun (gp2 vs io1)"},
                  {'flag': ["-p", "--ebs-iops"],
                   'help': "use a specified ebs iops for the rerun"},
+                 {'flag': ["-w", "--ebs-throughput"],
+                  'help': "use a specified ebs throughput (for gp3 type EBS) for the rerun"},
                  {'flag': ["-k", "--key-name"],
                   'help': "use a specified key name for the rerun"},
                  {'flag': ["-n", "--name"],
@@ -261,7 +273,10 @@ class Subcommands(object):
                  {'flag': ["-P", "--do-not-delete-public-access-block"],
                   'action': "store_true",
                   'help': "Do not delete public access block from buckets" +
-                          "(this way postrunjson and metrics reports will not be public)"}],
+                          "(this way postrunjson and metrics reports will not be public)"},
+                 {'flag': ["-q", "--quiet"],
+                  'action': "store_true",
+                  'help': "minimize standard output from deployment"}],
             'deploy_core':
                 [{'flag': ["-n", "--name"],
                   'help': "name of the lambda function to deploy (e.g. run_task_awsem)"},
@@ -270,7 +285,10 @@ class Subcommands(object):
                           "Lambda function, within the same usergroup"},
                  {'flag': ["-g", "--usergroup"],
                   'default': '',
-                  'help': "Tibanna usergroup for the AWS Lambda function"}],
+                  'help': "Tibanna usergroup for the AWS Lambda function"},
+                 {'flag': ["-q", "--quiet"],
+                  'action': "store_true",
+                  'help': "minimize standard output from deployment"}],
             'plot_metrics':
                 [{'flag': ["-j", "--job-id"],
                   'help': "job id of the specific job to log (alternative to --exec-arn/-e)"},
@@ -306,7 +324,10 @@ class Subcommands(object):
                   'help': "job id of the specific job to log (alternative to --exec-arn/-e)"},
                  {'flag': ["-u", "--update-tsv"],
                   'help': "update estimated cost in the metric tsv file on S3",
-                  'action': "store_true"}],
+                  'action': "store_true"},
+                  {'flag': ["-f", "--force"],
+                  'action': "store_true",
+                  'help': "returns the estimate, even if the actual cost is available"}],
             'cleanup':
                 [{'flag': ["-g", "--usergroup"],
                   'help': "Tibanna usergroup that shares the permission to access buckets and run jobs"},
@@ -347,11 +368,11 @@ class Subcommands(object):
         }
 
 
-def deploy_core(name, suffix=None, usergroup=''):
+def deploy_core(name, suffix=None, usergroup='', quiet=False):
     """
     New method of deploying packaged lambdas (BETA)
     """
-    API().deploy_core(name=name, suffix=suffix, usergroup=usergroup)
+    API().deploy_core(name=name, suffix=suffix, usergroup=usergroup, quiet=quiet)
 
 
 def run_workflow(input_json, sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, jobid='', do_not_open_browser=False, sleep=3):
@@ -374,11 +395,12 @@ def setup_tibanna_env(buckets='', usergroup_tag='default', no_randomize=False,
 
 
 def deploy_unicorn(suffix=None, no_setup=False, buckets='',
-                   no_setenv=False, usergroup='', do_not_delete_public_access_block=False, deploy_costupdater = False):
+                   no_setenv=False, usergroup='', do_not_delete_public_access_block=False,
+                   deploy_costupdater=False, quiet=False):
     """deploy tibanna unicorn to AWS cloud"""
     API().deploy_unicorn(suffix=suffix, no_setup=no_setup, buckets=buckets, no_setenv=no_setenv,
                          usergroup=usergroup, do_not_delete_public_access_block=do_not_delete_public_access_block,
-                         deploy_costupdater=deploy_costupdater)
+                         deploy_costupdater=deploy_costupdater, quiet=quiet)
 
 
 def add_user(user, usergroup):
@@ -403,14 +425,14 @@ def log(exec_arn=None, job_id=None, exec_name=None, sfn=TIBANNA_DEFAULT_STEP_FUN
                     top=top, top_latest=top_latest))
 
 
-def kill_all(sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME):
+def kill_all(sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, soft=False):
     """kill all the running jobs on a step function"""
-    API().kill_all(sfn)
+    API().kill_all(sfn, soft=soft)
 
 
-def kill(exec_arn=None, job_id=None, sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME):
+def kill(exec_arn=None, job_id=None, sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, soft=False):
     """kill a specific job"""
-    API().kill(exec_arn, job_id, sfn)
+    API().kill(exec_arn, job_id, sfn, soft=soft)
 
 
 def info(job_id):
@@ -419,19 +441,19 @@ def info(job_id):
 
 
 def rerun(exec_arn=None, job_id=None, sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, app_name_filter=None,
-          instance_type=None, shutdown_min=None, ebs_size=None, ebs_type=None, ebs_iops=None,
+          instance_type=None, shutdown_min=None, ebs_size=None, ebs_type=None, ebs_iops=None, ebs_throughput=None,
           overwrite_input_extra=None, key_name=None, name=None, use_spot=None, do_not_use_spot=None):
     """ rerun a specific job"""
     API().rerun(exec_arn=exec_arn, job_id=job_id, sfn=sfn,
                 app_name_filter=app_name_filter, instance_type=instance_type, shutdown_min=shutdown_min,
-                ebs_size=ebs_size, ebs_type=ebs_type, ebs_iops=ebs_iops,
+                ebs_size=ebs_size, ebs_type=ebs_type, ebs_iops=ebs_iops, ebs_throughput=ebs_throughput,
                 overwrite_input_extra=overwrite_input_extra, key_name=key_name, name=name,
                 use_spot=use_spot, do_not_use_spot=do_not_use_spot)
 
 
 def rerun_many(sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, stopdate='13Feb2018', stophour=13,
                stopminute=0, offset=0, sleeptime=5, status='FAILED', app_name_filter=None,
-               instance_type=None, shutdown_min=None, ebs_size=None, ebs_type=None, ebs_iops=None,
+               instance_type=None, shutdown_min=None, ebs_size=None, ebs_type=None, ebs_iops=None, ebs_throughput=None,
                overwrite_input_extra=None, key_name=None, name=None, use_spot=None, do_not_use_spot=None):
     """rerun all the jobs that failed after a given time point
     filtered by the time when the run failed (stopdate, stophour (24-hour format), stopminute)
@@ -448,7 +470,7 @@ def rerun_many(sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, stopdate='13Feb2018', sto
     API().rerun_many(sfn=sfn, stopdate=stopdate, stophour=stophour,
                      stopminute=stopminute, offset=offset, sleeptime=sleeptime, status=status,
                      app_name_filter=app_name_filter, instance_type=instance_type, shutdown_min=shutdown_min,
-                     ebs_size=ebs_size, ebs_type=ebs_type, ebs_iops=ebs_iops,
+                     ebs_size=ebs_size, ebs_type=ebs_type, ebs_iops=ebs_iops, ebs_throughput=ebs_throughput,
                      overwrite_input_extra=overwrite_input_extra, key_name=key_name, name=name,
                      use_spot=use_spot, do_not_use_spot=do_not_use_spot)
 
@@ -471,9 +493,9 @@ def cost(job_id, sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, update_tsv=False):
     """print out cost of a specific job"""
     print(API().cost(job_id=job_id, sfn=sfn, update_tsv=update_tsv))
 
-def cost_estimate(job_id, update_tsv=False):
+def cost_estimate(job_id, update_tsv=False, force=False):
     """print out estimated cost of a specific job"""
-    cost_estimate, cost_estimate_type = API().cost_estimate(job_id=job_id, update_tsv=update_tsv)
+    cost_estimate, cost_estimate_type = API().cost_estimate(job_id=job_id, update_tsv=update_tsv, force=force)
     print(f'{cost_estimate} ({cost_estimate_type})')
 
 
