@@ -287,79 +287,37 @@ class API(object):
         '''returns content from dynamodb for a given job id in a dictionary form'''
         return Job.info(job_id)
 
-    def kill(self, exec_arn=None, job_id=None, sfn=None):
-        sf = boto3.client('stepfunctions')
-        if exec_arn:
-            desc = sf.describe_execution(executionArn=exec_arn)
-            if desc['status'] == 'RUNNING':
-                jobid = str(json.loads(desc['input'])['jobid'])
-                ec2 = boto3.resource('ec2')
-                terminated = None
-                for i in ec2.instances.all():
-                    if i.tags:
-                        for tag in i.tags:
-                            if tag['Key'] == 'Type' and tag['Value'] != 'awsem':
-                                continue
-                            if tag['Key'] == 'Name' and tag['Value'] == 'awsem-' + jobid:
-                                logger.info("terminating EC2 instance")
-                                response = i.terminate()
-                                logger.info("Successfully terminated instance: " + str(response))
-                                terminated = True
-                                break
-                        if terminated:
-                            break
-                logger.info("terminating step function execution")
-                resp_sf = sf.stop_execution(executionArn=exec_arn, error="Aborted")
-                logger.info("Successfully terminated step function execution: " + str(resp_sf))
-        elif job_id:
+    def kill(self, exec_arn=None, job_id=None, sfn=None, soft=False):
+        """kills a running job, identified by either execution arn (exec_arn) or
+        job_id, (or job_id and sfn, for old tibanna versions).
+        This function kills both ec2 instance and the step function.
+        if soft is set True, it will not directly kill the step function
+        but instead sends a abort signal to S3 and let the step fucntion handle
+        the signal. The abort signal handling is available for >=1.3.0.
+        """
+        job = Job(exec_arn=exec_arn, job_id=job_id, sfn=sfn)
+        if job.check_status() == 'RUNNING':
+            # kill awsem ec2 instance
             ec2 = boto3.client('ec2')
-            res = ec2.describe_instances(Filters=[{'Name': 'tag:Name', 'Values': ['awsem-' + job_id]}])
+            res = ec2.describe_instances(Filters=[{'Name': 'tag:Name', 'Values': ['awsem-' + job.job_id]}])
             if not res['Reservations']:
-                raise("instance not available - if you just submitted the job, try again later")
+                raise Exception("instance not available - if you just submitted the job, try again later")
             instance_id = res['Reservations'][0]['Instances'][0]['InstanceId']
             logger.info("terminating EC2 instance")
             resp_term = ec2.terminate_instances(InstanceIds=[instance_id])
             logger.info("Successfully terminated instance: " + str(resp_term))
-            # first try dynanmodb to get logbucket
-            ddres = dict()
-            try:
-                dd = boto3.client('dynamodb')
-                ddres = dd.query(TableName=DYNAMODB_TABLE,
-                                 KeyConditions={'Job Id': {'AttributeValueList': [{'S': job_id}],
-                                                           'ComparisonOperator': 'EQ'}})
-            except Exception as e:
-                pass
-            if 'Items' in ddres:
-                exec_name = ddres['Items'][0]['Execution Name']['S']
-                sfn = ddres['Items'][0]['Step Function']['S']
-                exec_arn = EXECUTION_ARN(exec_name, sfn)
+
+            if soft:
+                logger.info("sending abort signal to s3")
+                s3 = boto3.client('s3')
+                s3.put_object(Body=b'', Bucket=job.log_bucket, Key=job.job_id + '.aborted')
+                logger.info("Successfully sent abort signal")
             else:
-                if not sfn:
-                    logger.warning("Can't stop step function because step function name is not given.")
-                    return None
-                stateMachineArn = STEP_FUNCTION_ARN(sfn)
-                res = sf.list_executions(stateMachineArn=stateMachineArn, statusFilter='RUNNING')
-                exec_arn = None
-                while True:
-                    if 'executions' not in res or not res['executions']:
-                        break
-                    for exc in res['executions']:
-                        desc = sf.describe_execution(executionArn=exc['executionArn'])
-                        if job_id == str(json.loads(desc['input'])['jobid']):
-                            exec_arn = exc['executionArn']
-                            break
-                    if exec_arn:
-                        break
-                    if 'nextToken' in res:
-                        res = sf.list_executions(nextToken=res['nextToken'],
-                                                 stateMachineArn=stateMachineArn, statusFilter='RUNNING')
-                    else:
-                        break
-                if not exec_arn:
-                    raise Exception("can't find the execution")
-            logger.info("terminating step function execution")
-            resp_sf = sf.stop_execution(executionArn=exec_arn, error="Aborted")
-            logger.info("Successfully terminated step function execution: " + str(resp_sf))
+                # kill step function execution
+                logger.info("terminating step function execution")
+                sf = boto3.client('stepfunctions')
+                resp_sf = sf.stop_execution(executionArn=job.exec_arn, error="Aborted")
+                logger.info("Successfully terminated step function execution: " + str(resp_sf))
 
     def kill_all(self, sfn=None):
         """killing all the running jobs"""
@@ -399,46 +357,9 @@ class API(object):
         sf = boto3.client('stepfunctions')
         if not exec_arn and exec_name:
             exec_arn = EXECUTION_ARN(exec_name, sfn)
-        if exec_arn:
-            desc = sf.describe_execution(executionArn=exec_arn)
-            job_id = str(json.loads(desc['input'])['jobid'])
-            if not logbucket:
-                logbucket = str(json.loads(desc['input'])['config']['log_bucket'])
-        elif job_id:
-            if not logbucket:
-                # first try dynanmodb to get logbucket
-                try:
-                    logbucket = self.info(job_id)['Log Bucket']
-                except Exception as e:
-                    pass
-                if not logbucket:
-                    # search through executions to get logbucket
-                    stateMachineArn = STEP_FUNCTION_ARN(sfn)
-                    try:
-                        res = sf.list_executions(stateMachineArn=stateMachineArn)
-                        while True:
-                            if 'executions' not in res or not res['executions']:
-                                break
-                            breakwhile = False
-                            for exc in res['executions']:
-                                desc = sf.describe_execution(executionArn=exc['executionArn'])
-                                if job_id == str(json.loads(desc['input'])['jobid']):
-                                    logbucket = str(json.loads(desc['input'])['config']['log_bucket'])
-                                    breakwhile = True
-                                    break
-                            if breakwhile:
-                                break
-                            if 'nextToken' in res:
-                                res = sf.list_executions(nextToken=res['nextToken'],
-                                                         stateMachineArn=stateMachineArn)
-                            else:
-                                break
-                    except:
-                        raise Exception("Cannot retrieve job. Try again later.")
-        else:
-            raise Exception("Either job_id, exec_arn or exec_name must be provided.")
+        job = Job(exec_arn=exec_arn, job_id=job_id, sfn=sfn)
         try:
-            res_s3 = boto3.client('s3').get_object(Bucket=logbucket, Key=job_id + suffix)
+            res_s3 = boto3.client('s3').get_object(Bucket=job.log_bucket, Key=job.job_id + suffix)
         except Exception as e:
             if 'NoSuchKey' in str(e):
                 if not quiet:
@@ -597,6 +518,8 @@ class API(object):
 
     def clear_input_json_template(self, input_json_template):
         """clear awsem template for reuse"""
+        if 'jobid' in input_json_template:
+            del input_json_template['jobid']
         if 'response' in input_json_template['_tibanna']:
             del(input_json_template['_tibanna']['response'])
         if 'run_name' in input_json_template['_tibanna'] and len(input_json_template['_tibanna']['run_name']) > 40:
@@ -711,7 +634,7 @@ class API(object):
             )
         return envlist.get(name, '')
 
-    def deploy_lambda(self, name, suffix, usergroup=''):
+    def deploy_lambda(self, name, suffix, usergroup='', quiet=False):
         """
         deploy a single lambda using the aws_lambda.deploy_function (BETA)
         """
@@ -760,20 +683,32 @@ class API(object):
             full_function_name = function_name_prefix
         if name not in self.do_not_delete:
             try:
-                boto3.client('lambda').get_function(FunctionName=full_function_name)
-                logger.info("deleting existing lambda")
-                boto3.client('lambda').delete_function(FunctionName=full_function_name)
+                if quiet:
+                    res = boto3.client('lambda').get_function(FunctionName=full_function_name)
+                    logger.info("deleting existing lambda")
+                    res = boto3.client('lambda').delete_function(FunctionName=full_function_name)
+                else:
+                    boto3.client('lambda').get_function(FunctionName=full_function_name)
+                    logger.info("deleting existing lambda")
+                    boto3.client('lambda').delete_function(FunctionName=full_function_name)
             except Exception as e:
                 if 'Function not found' in str(e):
                     pass
 
-        aws_lambda.deploy_function(lambda_fxn_module,
-                                   function_name_suffix=function_name_suffix,
-                                   package_objects=self.tibanna_packages,
-                                   requirements_fpath=requirements_fpath,
-                                   extra_config=extra_config)
+        if quiet:
+            res = aws_lambda.deploy_function(lambda_fxn_module,
+                                             function_name_suffix=function_name_suffix,
+                                             package_objects=self.tibanna_packages,
+                                             requirements_fpath=requirements_fpath,
+                                             extra_config=extra_config)
+        else:
+            aws_lambda.deploy_function(lambda_fxn_module,
+                                       function_name_suffix=function_name_suffix,
+                                       package_objects=self.tibanna_packages,
+                                       requirements_fpath=requirements_fpath,
+                                       extra_config=extra_config)
 
-    def deploy_core(self, name, suffix=None, usergroup=''):
+    def deploy_core(self, name, suffix=None, usergroup='', quiet=False):
         """deploy/update lambdas only"""
         logger.info("preparing for deploy...")
         if name == 'all':
@@ -783,7 +718,7 @@ class API(object):
         else:
             names = [name, ]
         for name in names:
-            self.deploy_lambda(name, suffix, usergroup)
+            self.deploy_lambda(name, suffix, usergroup, quiet=quiet)
 
     def setup_tibanna_env(self, buckets='', usergroup_tag='default', no_randomize=False,
                           do_not_delete_public_access_block=False, verbose=False):
@@ -819,7 +754,7 @@ class API(object):
 
     def deploy_tibanna(self, suffix=None, usergroup='', setup=False,
                        buckets='', setenv=False, do_not_delete_public_access_block=False,
-                       deploy_costupdater=False):
+                       deploy_costupdater=False, quiet=False):
         """deploy tibanna unicorn or pony to AWS cloud (pony is for 4DN-DCIC only)"""
         if setup:
             if usergroup:
@@ -842,21 +777,22 @@ class API(object):
                 outfile.write("\nexport TIBANNA_DEFAULT_STEP_FUNCTION_NAME=%s\n" % step_function_name)
 
         logger.info("deploying lambdas...")
-        self.deploy_core('all', suffix=suffix, usergroup=usergroup)
+        self.deploy_core('all', suffix=suffix, usergroup=usergroup, quiet=quiet)
 
         if(deploy_costupdater):
-            self.deploy_lambda(self.update_cost_lambda, suffix=suffix, usergroup=usergroup)
+            self.deploy_lambda(self.update_cost_lambda, suffix=suffix, usergroup=usergroup, quiet=quiet)
 
         dd_utils.create_dynamo_table(DYNAMODB_TABLE, DYNAMODB_KEYNAME)
         return step_function_name
 
     def deploy_unicorn(self, suffix=None, no_setup=False, buckets='',
-                       no_setenv=False, usergroup='', do_not_delete_public_access_block=False, deploy_costupdater = False):
+                       no_setenv=False, usergroup='', do_not_delete_public_access_block=False,
+                       deploy_costupdater=False, quiet=False):
         """deploy tibanna unicorn to AWS cloud"""
         self.deploy_tibanna(suffix=suffix, usergroup=usergroup, setup=not no_setup,
                             buckets=buckets, setenv=not no_setenv,
                             do_not_delete_public_access_block=do_not_delete_public_access_block,
-                            deploy_costupdater=deploy_costupdater)
+                            deploy_costupdater=deploy_costupdater, quiet=quiet)
 
     def add_user(self, user, usergroup):
         """add a user to a tibanna group"""
@@ -1046,7 +982,7 @@ class API(object):
         if open_browser:
             webbrowser.open(METRICS_URL(log_bucket, job_id))
 
-    def cost_estimate(self, job_id, update_tsv=False):
+    def cost_estimate(self, job_id, update_tsv=False, force=False):
         postrunjsonstr = self.log(job_id=job_id, postrunjson=True)
         if not postrunjsonstr:
             logger.info("Cost estimation error: postrunjson not found")
@@ -1055,12 +991,13 @@ class API(object):
         postrunjson = AwsemPostRunJson(**postrunjsonobj)
         log_bucket = postrunjson.config.log_bucket
 
-        # We return the real cost, if it is availble, but don't automatically update the Cost row in the tsv
-        precise_cost = self.cost(job_id, update_tsv=False)
-        if(precise_cost and precise_cost > 0.0):
-            if update_tsv:
-                update_cost_estimate_in_tsv(log_bucket, job_id, precise_cost, cost_estimate_type="actual cost")
-            return precise_cost, "actual cost"
+        # We return the real cost, if it is available, but don't automatically update the Cost row in the tsv
+        if not force:
+            precise_cost = self.cost(job_id, update_tsv=False)
+            if(precise_cost and precise_cost > 0.0):
+                if update_tsv:
+                    update_cost_estimate_in_tsv(log_bucket, job_id, precise_cost, cost_estimate_type="actual cost")
+                return precise_cost, "actual cost"
 
         # awsf_image was added in 1.0.0. We use that to get the correct ebs root type
         ebs_root_type = 'gp3' if 'awsf_image' in postrunjsonobj['config'] else 'gp2'
